@@ -6,6 +6,7 @@
 #include <string.h>
 #include <ctype.h>
 #include <stdio.h>
+#include <limits.h>
 
 #ifdef _WIN32_WCE
 #include <windows.h>
@@ -25,26 +26,40 @@ typedef long long __int64;
 #define M_PI 3.14159265358979323846 // Not in math ??
 #endif
 
+#if __FreeBSD__ || __APPLE__  // Thanks to Ted Mielczarek & Dmitry
+#define fopen64(x,y) fopen(x,y)
+#endif
+
 #ifndef TRUE
 #define TRUE 1
 #define FALSE 0
 #endif
 
+// this definitions have been changes to prevent compiler warnings
+// copied verbatim for /usr/include/limits.h
+#ifndef INT_MAX
+#define INT_MAX       2147483647
+#endif
+
 #ifndef INT_MIN
-#define INT_MIN 0x80000000 // -2147483648
+#define INT_MIN       (-INT_MAX - 1)
 #endif
 
 #define Sqr(x) ((x)*(x))
+inline int isqrt (int x) { return lrint (sqrt (x)); } // Optimize this ?
 
 #define TILEBITS (18)
 #define TILESIZE (1<<TILEBITS)
 
 #define RESTRICTIONS M (access) M (motorcar) M (bicycle) M (foot) M (goods) \
-  M (hgv) M (horse) M (motorcycle) M (psv) M (motorboat) M (boat) \
+  M (hgv) M (horse) M (motorcycle) M (psv) M (moped) M (mofa) \
+  M (motorboat) M (boat) \
   M (oneway) M (roundabout)
 
 #define M(field) field ## R,
-enum { STYLE_BITS = 8, RESTRICTIONS layerBit1,layerBit2,layerBit3 };
+enum {
+  STYLE_BITS = 10, RESTRICTIONS bicycleOneway, layerBit1, layerBit2, layerBit3
+};
 #undef M
 
 // Below is a list of the tags that the user may add. It should also include
@@ -215,6 +230,7 @@ enum { STYLE_BITS = 8, RESTRICTIONS layerBit1,layerBit2,layerBit3 };
  s (tourism, caravan_site,    "caravan site"    , "") \
  s (amenity, pharmacy,        "pharmacy"        , "") \
  s (amenity, dentist,         "dentist"         , "") \
+ s (amenity, doctor,          "doctor"          , "") \
  s (amenity, hospital,        "hospital"        , "") \
  s (amenity, bank,            "bank"            , "") \
  s (amenity, bureau_de_change, "bureau de change" , "") \
@@ -278,11 +294,13 @@ enum { STYLE_BITS = 8, RESTRICTIONS layerBit1,layerBit2,layerBit3 };
                "  <tag k='amenity' v='place_of_worship' />\n") \
  s (religion, taoist,         "taoism"          , \
                "  <tag k='amenity' v='place_of_worship' />\n") \
+ s (highway,         road,     "hway=road"      , "") \
  /* relations must be last and restriction_no_right_turn must be first */ \
  s (restriction, no_right_turn, ""              , "") \
  s (restriction, no_left_turn, ""               , "") \
  s (restriction, no_u_turn, ""                  , "") \
  s (restriction, no_straight_on, ""             , "") \
+ /* restriction_no_straight_on must be the last "no_" restriction */ \
  s (restriction, only_right_turn, ""            , "") \
  s (restriction, only_left_turn, ""             , "") \
  s (restriction, only_straight_on, ""           , "") \
@@ -306,11 +324,25 @@ struct styleStruct {
 struct ndType {
   int wayPtr, lat, lon, other[2];
 };
+/* This struct takes up a lot of space, but compressing is possible: If
+other is encoded as byte offset from the current position, it should typically
+be close to 0. So it can be Huffman encoded. Then the struct will no longer
+have a fixed size, and the final position of an nd will not be known during
+the first pass. So we over estimate the space needed between 2 nds and we can
+do multiple passes until all the excess has been removed.
+
+The difference between lat and wayPtr->clat will normally also be quite
+small and can also be Huffman coded. The same applies to lon.
+
+C code can be generated for these non dynamic Huffman decoders and the C
+compiler can then optimize it for efficient decompression. Not only will this
+save (flash) storage, but it will also increase the size of maps that can be
+loaded and reduce the paging that is so slow on mobile devices.
+*/
 
 struct wayType {
   int bits, destination;
   int clat, clon, dlat, dlon; /* Centre coordinates and (half)diameter */
-  float maxspeed;
 };
 
 inline int Layer (wayType *w) { return w->bits >> 29; }
@@ -342,8 +374,12 @@ inline double LonInverse (int lon)
 extern wayType *gosmSway[searchCnt];
 extern int *hashTable, bucketsMin1, pakHead;
 extern char *gosmData, *gosmSstr[searchCnt];
+/* gosmSstr is no longer nul terminated. If it's a problem, use
+string s (gosmSstr[i], strcspn (gosmSstr[i], "\n")); ... s.c_str() ... */
+
 extern ndType *ndBase;
 extern styleStruct *style;
+extern int stylecount;
 
 inline wayType *Way (ndType *nd) { return (wayType *)(nd->wayPtr+gosmData); }
 
@@ -383,7 +419,7 @@ inline int Hash (int lon, int lat, int lowz = 0)
     (lowz ? bucketsMin1 >> 7 : bucketsMin1)) + (lowz ? bucketsMin1 + 1 : 0);
 }
 
-int TagCmp (char *a, char *b); 
+// int TagCmp (const char *a, const char *b); // Only used in libgosm
 
 struct OsmItr { // Iterate over all the objects in a square
   ndType *nd[1]; /* Readonly. Either can be 'from' or 'to', but you */
@@ -394,7 +430,7 @@ struct OsmItr { // Iterate over all the objects in a square
   
   OsmItr (int l, int t, int r, int b)
   {
-    tsize = r - l > 10000000 ? TILESIZE << 7 : TILESIZE;
+    tsize = r - l > 1500000 ? TILESIZE << 7 : TILESIZE;
     left = l & (~(tsize - 1));
     right = (r + tsize - 1) & (~(tsize-1));
     top = t & (~(tsize - 1));
@@ -411,17 +447,30 @@ int Next (OsmItr &itr); /* Friend of osmItr */
 struct routeNodeType {
   ndType *nd;
   routeNodeType *shortest;
-  int best, heapIdx, dir, remain; // Dir is 0 or 1
+  int heapIdx, dir, remain; // Dir is 0 or 1
+  // if heapIdx is negative, the node is not in the heap and best = -heapIdx.
+};
+/* The data is split over two structures (routeNodeType and routeHeapType).
+  Some of these fields may cause fewer cache misses if their are in
+  routeHeapType */
+struct routeHeapType {
+  routeNodeType *r;
+  int best;
 };
 
-extern routeNodeType *route, *shortest, **routeHeap;
-extern int routeHeapSize, tlat, tlon, flat, flon, rlat, rlon;
+extern routeNodeType *route, *shortest;
+extern routeHeapType *routeHeap;
+extern int routeHeapSize, tlat, tlon, flat, flon, rlat, rlon, routeSuccess;
 
 void Route (int recalculate, int plon, int plat, int Vehicle, int fast);
+int RouteLoop (void);
+void GosmFreeRoute (void);
 
 int JunctionType (ndType *nd);
 
-void GosmSearch (int clon, int clat, char *key);
+int *GosmIdxSearch (const char *key, unsigned z);
+// GosmIdxSearch is only exported in order to find unique object, like cities.
+void GosmSearch (int clon, int clat, const char *key);
 
 int GosmInit (void *d, long size);
 
@@ -445,15 +494,17 @@ typedef struct {
 // speeds for each vehicle type in maxspeeds. styleCnt representing
 // the location of the first elemstyle. Returns the final styleCnt.
 int LoadElemstyles(/* in */ const char *elemstylesfname, 
-		   const char *iconsfname, int styleCnt,
-		   /* out */ styleStruct *srec, elemstyleMapping *map, 
-		   float *maxspeeds);
+		   const char *iconsfname,
+		   /* out */ styleStruct *srec, elemstyleMapping *map);
 
 // creates a new pakfile from an osmdata file read from standard in
 int RebuildPak(const char* pakfile, const char* elemstylefile, 
 	       const char* iconscsvfile, const char* masterpakfile, 
 	       const int bbox[4]);
+int SortRelations (void);
 
 #endif // #ifndef _WIN32
 
+void logprintf(const char *format, ...);
+
 #endif
